@@ -2113,9 +2113,7 @@ static int parser_init (parser_object* self, PyObject* args, PyObject* kwds) {
 
 /* traverse all used subobjects participating in reference cycles */
 static int parser_traverse (parser_object* self, visitproc visit, void* arg) {
-    if (visit(self->handler, arg) < 0) {
-        return -1;
-    }
+    Py_VISIT(self->handler);
     return 0;
 }
 
@@ -2193,8 +2191,8 @@ static PyObject* parser_flush (parser_object* self, PyObject* args) {
         int error = 0;
         const char* enc = PyString_AsString(self->encoding);
 	PyObject* s = PyUnicode_Decode(self->userData->buf,
-                                       strlen(self->userData->buf),
-                                       enc, "ignore");
+                                     (Py_ssize_t)strlen(self->userData->buf),
+                                     enc, "ignore");
 	PyObject* callback = NULL;
 	PyObject* result = NULL;
 	/* reset buffer */
