@@ -31,6 +31,13 @@
 #error please install Python >= 2.4
 #endif
 
+/* See http://www.python.org/dev/peps/pep-0353/#conversion-guidelines */
+#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
+typedef int Py_ssize_t;
+#define PY_SSIZE_T_MAX INT_MAX
+#define PY_SSIZE_T_MIN INT_MIN
+#endif
+
 /* user_data type for SAX calls */
 typedef struct {
     /* the Python SAX object to issue callbacks */
