@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2005  Bastian Kleineidam
+# Copyright (C) 2000-2006 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ Handle http links.
 """
 
 import urlparse
+import urllib
 import time
 import re
 import zlib
@@ -26,7 +27,6 @@ import socket
 import cStringIO as StringIO
 import Cookie
 
-import linkcheck
 import linkcheck.url
 import linkcheck.gzip2
 import linkcheck.strformat
@@ -36,6 +36,9 @@ import httpheaders as headers
 import internpaturl
 import proxysupport
 
+# helper alias
+unicode_safe = linkcheck.strformat.unicode_safe
+
 supportHttps = hasattr(linkcheck.httplib2, "HTTPSConnection") and \
                hasattr(socket, "ssl")
 
@@ -128,8 +131,9 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         """
         roboturl = self.get_robots_txt_url()
         user, password = self.get_user_password()
-        return self.consumer.robots_txt_allows_url(roboturl, url,
-                                                   user, password)
+        rb = self.aggregate.robots_txt
+        callback = self.aggregate.connections.host_wait
+        return rb.allows_url(roboturl, url, user, password, callback=callback)
 
     def check_connection (self):
         """
@@ -149,7 +153,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             valid request
         """
         # set the proxy, so a 407 status after this is an error
-        self.set_proxy(self.consumer.config("proxy").get(self.scheme))
+        self.set_proxy(self.aggregate.config["proxy"].get(self.scheme))
         # initialize check data
         self.headers = None
         self.auth = None
@@ -177,10 +181,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         else:
             server = _("unknown")
         if self.fallback_get:
-            self.add_info(_("Server %r did not support HEAD request; "\
+            self.add_info(_("Server %r did not support HEAD request; "
                             "a GET request was used instead.") % server)
         if self.no_anchor:
-            self.add_warning(_("Server %r had no anchor support, removed"\
+            self.add_warning(_("Server %r had no anchor support, removed"
                                " anchor from request.") % server,
                              tag="http-no-anchor-support")
         # redirections might have changed the URL
@@ -190,6 +194,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # check response
         if response:
             self.check_response(response)
+            response.close()
 
     def check_http_connection (self):
         """
@@ -199,7 +204,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         @return: response or None if url is already handled
         @rtype: HttpResponse or None
         """
+        response = None
         while True:
+            if response is not None:
+                response.close()
             try:
                 response = self._get_http_response()
             except linkcheck.httplib2.BadStatusLine:
@@ -211,12 +219,11 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     continue
                 raise
             if response.reason:
-                response.reason = \
-                        linkcheck.strformat.unicode_safe(response.reason)
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "Response: %s %s",
-                                response.status, response.reason)
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "Headers: %s",
-                                self.headers)
+                response.reason = unicode_safe(response.reason)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "Response: %s %s", response.status, response.reason)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "Headers: %s", self.headers)
             # proxy enforcement (overrides standard proxy)
             if response.status == 305 and self.headers:
                 oldproxy = (self.proxy, self.proxyauth)
@@ -228,6 +235,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                          _("Enforced proxy %r ignored, aborting.") % newproxy,
                          valid=False)
                     return response
+                response.close()
                 response = self._get_http_response()
                 # restore old proxy settings
                 self.proxy, self.proxyauth = oldproxy
@@ -242,7 +250,9 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     continue
                 raise
             if tries == -1:
-                linkcheck.log.debug(linkcheck.LOG_CHECK, "already handled")
+                assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                    "already handled")
+                response.close()
                 return None
             if tries >= self.max_redirects:
                 if self.method == "HEAD":
@@ -251,7 +261,7 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     self.aliases = []
                     self.fallback_get = True
                     continue
-                self.set_result(_("more than %d redirections, aborting") % \
+                self.set_result(_("more than %d redirections, aborting") %
                                 self.max_redirects, valid=False)
                 return response
             # user authentication
@@ -259,10 +269,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                 if not self.auth:
                     import base64
                     _user, _password = self.get_user_password()
-                    self.auth = "Basic "+\
+                    self.auth = "Basic " + \
                         base64.encodestring("%s:%s" % (_user, _password))
-                    linkcheck.log.debug(linkcheck.LOG_CHECK,
-                                    "Authentication %s/%s", _user, _password)
+                    assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                        "Authentication %s/%s", _user, _password)
                     continue
             elif response.status >= 400:
                 if self.headers and self.urlparts[4] and not self.no_anchor:
@@ -288,11 +298,12 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             break
         return response
 
-    def follow_redirections (self, response):
+    def follow_redirections (self, response, set_result=True):
         """
         Follow all redirections of http response.
         """
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "follow all redirections")
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "follow all redirections")
         redirected = self.url
         tries = 0
         while response.status in [301, 302] and self.headers and \
@@ -301,36 +312,33 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                          self.headers.getheader("Uri", ""))
             # make new url absolute and unicode
             newurl = urlparse.urljoin(redirected, newurl)
-            newurl = linkcheck.strformat.unicode_safe(newurl)
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "Redirected to %r",
-                                newurl)
-            self.add_info(_("Redirected to %(url)s.") % {'url': newurl})
-            redirected, is_idn = linkcheck.url.url_norm(newurl)
-            linkcheck.log.debug(linkcheck.LOG_CHECK, "Norm redirected to %r",
-                                redirected)
+            newurl = unicode_safe(newurl)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "Redirected to %r", newurl)
+            self.add_info(_("Redirected to %(url)s.") % {'url': newurl},
+                          tag="http-redirect")
+            # norm base url - can raise UnicodeError from url.idna_encode()
+            redirected, is_idn = linkcheck.checker.urlbase.url_norm(newurl)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "Norm redirected to %r", redirected)
             urlparts = linkcheck.strformat.url_unicode_split(redirected)
-            # check if we still have the same scheme type, it could be a
-            # different one
-            if urlparts[0] != self.scheme:
-                self.add_warning(
-                           _("Redirection to different URL type encountered; "
-                             "the original URL was %r.") % self.url,
-                           tag="http-wrong-redirect")
             # check extern filter again
             self.set_extern(redirected)
             if self.extern[0] and self.extern[0]:
-                self.add_info(
-                          _("The redirected URL is outside of the domain " \
+                if set_result:
+                    self.add_info(
+                          _("The redirected URL is outside of the domain "
                             "filter, checked only syntax."))
-                self.set_result(u"filtered")
+                    self.set_result(u"filtered")
                 return -1, response
             # check robots.txt allowance again
             if not self.allows_robots(redirected):
-                self.add_warning(
-                       _("Access to redirected URL denied by robots.txt, " \
+                if set_result:
+                    self.add_warning(
+                       _("Access to redirected URL denied by robots.txt, "
                          "checked only syntax."),
                        tag="http-robots-denied")
-                self.set_result(u"syntax OK")
+                    self.set_result(u"syntax OK")
                 return -1, response
             # see about recursive redirect
             all_seen = [self.cache_url_key] + self.aliases
@@ -340,42 +348,61 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                     # fall back to the original url and use GET
                     return self.max_redirects, response
                 recursion = all_seen + [redirected]
-                self.set_result(
-                          _("recursive redirection encountered:\n %s") % \
+                if set_result:
+                    self.set_result(
+                          _("recursive redirection encountered:\n %s") %
                             "\n  => ".join(recursion), valid=False)
                 return -1, response
-            # remember redireced url as alias
-            self.aliases.append(redirected)
+            if urlparts[0] == self.scheme:
+                # remember redireced url as alias
+                self.aliases.append(redirected)
             # note: urlparts has to be a list
             self.urlparts = urlparts
             if response.status == 301:
                 if not self.has301status:
-                    self.add_warning(
+                    if set_result:
+                        self.add_warning(
                            _("HTTP 301 (moved permanent) encountered: you"
                              " should update this link."),
                            tag="http-moved-permanent")
                     self.has301status = True
             # check cache again on the changed URL
-            if self.consumer.checked_redirect(redirected, self):
+            if self.aggregate.urlqueue.checked_redirect(redirected, self):
                 return -1, response
             # in case of changed scheme make new URL object
             if self.urlparts[0] != self.scheme:
+                if set_result:
+                    self.add_warning(
+                           _("Redirection to different URL type encountered; "
+                             "the original URL was %r.") % self.url,
+                           tag="http-wrong-redirect")
                 newobj = linkcheck.checker.get_url_from(
-                          redirected, self.recursion_level, self.consumer,
+                          redirected, self.recursion_level, self.aggregate,
                           parent_url=self.parent_url, base_ref=self.base_ref,
                           line=self.line, column=self.column, name=self.name,
-                          cmdline=False)
-                newobj.warnings = self.warnings
-                newobj.info = self.info
+                          assume_local=False)
                 # append new object to queue
-                self.consumer.append_url(newobj)
+                self.aggregate.urlqueue.put(newobj)
                 # pretend to be finished and logged
                 return -1, response
             # new response data
+            response.close()
             response = self._get_http_response()
             tries += 1
         return tries, response
 
+    def get_alias_cache_data (self):
+        """
+        Return all data values that should be put in the cache,
+        minus redirection warnings.
+        """
+        data = self.get_cache_data()
+        warns = [x for x in self.warnings if x[0] != "http-moved-permanent"]
+        data["warnings"] = warns
+        infos = [x for x in self.info if x[0] != "http-redirect"]
+        data["info"] = infos
+        return data
+
     def check_response (self, response):
         """
         Check final result and log it.
@@ -386,18 +413,19 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         else:
             if response.status == 204:
                 # no content
-                self.add_warning(
-                            linkcheck.strformat.unicode_safe(response.reason),
-                            tag="http-empty-content")
+                self.add_warning(unicode_safe(response.reason),
+                                 tag="http-empty-content")
             # store cookies for valid links
-            if self.consumer.config('cookies'):
+            if self.aggregate.config['storecookies']:
                 for c in self.cookies:
                     self.add_info(_("Store cookie: %s.") % c)
                 try:
-                    out = self.consumer.store_cookies(self.headers,
-                                                      self.urlparts[1])
+                    out = self.aggregate.cookies.add(self.headers,
+                                                     self.urlparts[0],
+                                                     self.urlparts[1],
+                                                     self.urlparts[2])
                     for h in out:
-                        self.add_info(linkcheck.strformat.unicode_safe(h))
+                        self.add_info(unicode_safe(h))
                 except Cookie.CookieError, msg:
                     self.add_warning(_("Could not store cookies: %(msg)s.") %
                                      {'msg': str(msg)},
@@ -420,7 +448,8 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         else:
             host = self.urlparts[1]
             scheme = self.urlparts[0]
-        linkcheck.log.debug(linkcheck.LOG_CHECK, "Connecting to %r", host)
+        assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+            "Connecting to %r", host)
         if self.url_connection:
             self.close_connection()
         self.url_connection = self.get_http_object(host, scheme)
@@ -452,14 +481,20 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                                       linkcheck.configuration.UserAgent)
         self.url_connection.putheader("Accept-Encoding",
                                   "gzip;q=1.0, deflate;q=0.9, identity;q=0.5")
-        if self.consumer.config('cookies'):
-            self.cookies = self.consumer.get_cookies(self.urlparts[1],
-                                                     self.urlparts[2])
+        if self.aggregate.config['sendcookies']:
+            scheme = self.urlparts[0]
+            host = self.urlparts[1]
+            port = linkcheck.url.default_ports.get(scheme, 80)
+            host, port = urllib.splitnport(host, port)
+            path = self.urlparts[2]
+            self.cookies = self.aggregate.cookies.get(scheme, host, port, path)
             for c in self.cookies:
-                self.url_connection.putheader("Cookie", c)
+                name = c.client_header_name()
+                value = c.client_header_value()
+                self.url_connection.putheader(name, value)
         self.url_connection.endheaders()
         response = self.url_connection.getresponse()
-        self.persistent = headers.http_persistent(response)
+        self.persistent = not response.will_close
         self.timeout = headers.http_timeout(response)
         self.headers = response.msg
         # If possible, use official W3C HTTP response name
@@ -480,18 +515,19 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         """
         _user, _password = self.get_user_password()
         key = (scheme, self.urlparts[1], _user, _password)
-        conn = self.consumer.get_connection(key)
+        conn = self.aggregate.connections.get(key)
         if conn is not None:
-            linkcheck.log.debug(linkcheck.LOG_CHECK,
-                                "reuse cached HTTP(S) connection %s", conn)
+            assert None == linkcheck.log.debug(linkcheck.LOG_CHECK,
+                "reuse cached HTTP(S) connection %s", conn)
             return conn
+        self.aggregate.connections.wait_for_host(host)
         if scheme == "http":
             h = linkcheck.httplib2.HTTPConnection(host)
         elif scheme == "https" and supportHttps:
             h = linkcheck.httplib2.HTTPSConnection(host)
         else:
-            raise linkcheck.LinkCheckerError, \
-                                 _("Unsupported HTTP url scheme %r") % scheme
+            msg = _("Unsupported HTTP url scheme %r") % scheme
+            raise linkcheck.LinkCheckerError(msg)
         if linkcheck.log.is_debug(linkcheck.LOG_CHECK):
             h.set_debuglevel(1)
         h.connect()
@@ -507,10 +543,11 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         """
         if not self.has_content:
             self.method = "GET"
-            self.has_content = True
             self.close_connection()
             t = time.time()
             response = self._get_http_response()
+            tries, response = self.follow_redirections(response,
+                                                       set_result=False)
             self.headers = response.msg
             self.data = response.read()
             encoding = headers.get_content_encoding(self.headers)
@@ -522,12 +559,14 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
                         f = linkcheck.gzip2.GzipFile('', 'rb', 9,
                                           StringIO.StringIO(self.data))
                 except zlib.error, msg:
-                    self.add_warning(_("Decompress error %(err)s") % \
+                    self.add_warning(_("Decompress error %(err)s") %
                                      {"err": str(msg)},
                                      tag="http-decompress-error")
                     f = StringIO.StringIO(self.data)
                 self.data = f.read()
             self.downloadtime = time.time() - t
+            response.close()
+            self.has_content = True
         return self.data
 
     def is_html (self):
@@ -588,7 +627,6 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
             self.parse_html()
         elif ptype == "text/css":
             self.parse_css()
-        return None
 
     def get_robots_txt_url (self):
         """
@@ -610,11 +648,10 @@ class HttpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         # add to cached connections
         _user, _password = self.get_user_password()
         key = ("http", self.urlparts[1], _user, _password)
-        cache_add = self.consumer.add_connection
-        # note: only cache the connection when it is persistent
-        # and all pending content has been received
-        if not self.persistent or not self.has_content or \
-           not cache_add(key, self.url_connection, self.timeout):
+        if self.persistent and self.url_connection.is_idle():
+            cache_add = self.aggregate.connections.add
+            cache_add(key, self.url_connection, self.timeout)
+        else:
             try:
                 self.url_connection.close()
             except:
