@@ -43,8 +43,8 @@ class SetList (list):
         """Set new value, and eliminate a possible duplicate value."""
         # search index i with self[i] == item
         delidx = -1
-        for i in xrange(len(self)):
-            if self[i] == item and i != index:
+        for i, x in enumerate(self):
+            if x == item and i != index:
                 delidx = i
                 # stop here, there can be only one duplicate
                 break
@@ -67,6 +67,11 @@ class ListDict (dict):
         # sorted list of keys
         self._keys = []
 
+    def setdefault (self, key, *args):
+        if not self.has_key(key):
+            self._keys.append(key)
+        return super(ListDict, self).setdefault(key, *args)
+
     def __setitem__ (self, key, value):
         """Add key,value to dict, append key to sorted list."""
         if not self.has_key(key):
@@ -78,6 +83,19 @@ class ListDict (dict):
         self._keys.remove(key)
         super(ListDict, self).__delitem__(key)
 
+    def pop (self, key):
+        if key in self._keys:
+            self._keys.remove(key)
+        super(ListDict, self).pop(key)
+
+    def popitem (self):
+        if self._keys:
+            k = self._keys[0]
+            v = self[k]
+            del self[k]
+            return (k, v)
+        raise KeyError("popitem() on empty dictionary")
+
     def values (self):
         """Return sorted list of values."""
         return [self[k] for k in self._keys]
@@ -92,15 +110,17 @@ class ListDict (dict):
 
     def itervalues (self):
         """Return iterator over sorted values."""
-        return iter(self.values())
+        for k in self._keys:
+            yield self[k]
 
     def iteritems (self):
         """Return iterator over sorted items."""
-        return iter(self.items())
+        for k in self._keys:
+            yield (k, self[k])
 
     def iterkeys (self):
         """Return iterator over sorted keys."""
-        return iter(self.keys())
+        return iter(self._keys)
 
     def clear (self):
         """Remove all dict entries."""
@@ -176,3 +196,6 @@ class CaselessSortedDict (CaselessDict):
 
     def items (self):
         return [(x, self[x]) for x in self.keys()]
+
+    def iteritems (self):
+        return ((x, self[x]) for x in self.keys())
