@@ -271,7 +271,7 @@ class HTTPResponse:
     # See RFC 2616 sec 19.6 and RFC 1945 sec 6 for details.
 
     def __init__(self, sock, debuglevel=0, strict=0, method=None):
-        self.fp = sock.makefile('rb', 0)
+        self.fp = sock.makefile('rb')
         self.debuglevel = debuglevel
         self.strict = strict
         self._method = method
@@ -288,6 +288,10 @@ class HTTPResponse:
         self.length = _UNKNOWN          # number of bytes left in response
         self.will_close = _UNKNOWN      # conn will close at end of response
 
+    def __str__ (self):
+        return "<HTTPResponse status=%s reason=%s chunked=%s length=%s>" % \
+          (self.status, self.reason, self.chunked, self.length)
+
     def _read_status(self):
         # Initialize with Simple-Response defaults
         line = self.fp.readline()
@@ -296,7 +300,7 @@ class HTTPResponse:
         if not line:
             # Presumably, the server closed the connection before
             # sending a valid response.
-            raise BadStatusLine(line)
+            raise BadStatusLine("Bad status line %r" % line)
         try:
             [version, status, reason] = line.split(None, 2)
         except ValueError:
@@ -310,7 +314,7 @@ class HTTPResponse:
         if not version.startswith('HTTP/'):
             if self.strict:
                 self.close()
-                raise BadStatusLine(line)
+                raise BadStatusLine("Bad status line %r" % line)
             else:
                 # assume it's a Simple-Response from an 0.9 server
                 self.fp = LineAndFileWrapper(line, self.fp)
@@ -320,9 +324,9 @@ class HTTPResponse:
         try:
             status = int(status)
             if status < 100 or status > 999:
-                raise BadStatusLine(line)
+                raise BadStatusLine("Bad status line %r" % line)
         except ValueError:
-            raise BadStatusLine(line)
+            raise BadStatusLine("Bad status line %r" % line)
         return version, status, reason
 
     def begin(self):
@@ -557,13 +561,13 @@ class HTTPResponse:
 
     def getheader(self, name, default=None):
         if self.msg is None:
-            raise ResponseNotReady()
+            raise ResponseNotReady("Response msg is None")
         return self.msg.getheader(name, default)
 
     def getheaders(self):
         """Return list of (header, value) tuples."""
         if self.msg is None:
-            raise ResponseNotReady()
+            raise ResponseNotReady("Response msg is None")
         return self.msg.items()
 
 
@@ -589,6 +593,11 @@ class HTTPConnection:
         if strict is not None:
             self.strict = strict
 
+    def __str__ (self):
+        return "<HttpConnection state=%s\n method=%s\n" \
+               " sock=%s\n buffer=%s\n response=%s>" % \
+        (self.__state, self._method, self.sock, self._buffer, self.__response)
+
     def _set_hostport(self, host, port):
         if port is None:
             i = host.rfind(':')
@@ -622,9 +631,9 @@ class HTTPConnection:
             except socket.error, msg:
                 if self.debuglevel > 0:
                     print 'connect fail:', (self.host, self.port)
-                if self.sock:
+                if self.sock is not None:
                     self.sock.close()
-                self.sock = None
+                    self.sock = None
                 continue
             break
         if not self.sock:
@@ -646,7 +655,7 @@ class HTTPConnection:
             if self.auto_open:
                 self.connect()
             else:
-                raise NotConnected()
+                raise NotConnected("cannot send when not connected")
 
         # send the data to the server. if we get a broken pipe, then close
         # the socket. we want to reconnect when somebody tries to send again.
@@ -656,9 +665,17 @@ class HTTPConnection:
         if self.debuglevel > 0:
             print "send:", repr(str)
         try:
-            self.sock.sendall(str)
+            blocksize=8192
+            if hasattr(str,'read') :
+                if self.debuglevel > 0: print "sendIng a read()able"
+                data=str.read(blocksize)
+                while data:
+                    self.sock.sendall(data)
+                    data=str.read(blocksize)
+            else:
+                self.sock.sendall(str)
         except socket.error, v:
-            if v[0] == errno.EPIPE:
+            if v[0] == 32:      # Broken pipe
                 self.close()
             raise
 
@@ -715,7 +732,7 @@ class HTTPConnection:
         if self.__state == _CS_IDLE:
             self.__state = _CS_REQ_STARTED
         else:
-            raise CannotSendRequest()
+            raise CannotSendRequest("cannot send request in state %s" % self.__state)
 
         # Save the method we use, we need it later in the response phase
         self._method = method
@@ -792,7 +809,7 @@ class HTTPConnection:
         For example: h.putheader('Accept', 'text/html')
         """
         if self.__state != _CS_REQ_STARTED:
-            raise CannotSendHeader()
+            raise CannotSendHeader("cannot send request in state %s" % self.__state)
 
         str = '%s: %s' % (header, value)
         self._output(str)
@@ -803,7 +820,7 @@ class HTTPConnection:
         if self.__state == _CS_REQ_STARTED:
             self.__state = _CS_REQ_SENT
         else:
-            raise CannotSendHeader()
+            raise CannotSendHeader("cannot send request in state %s" % self.__state)
 
         self._send_output()
 
@@ -831,7 +848,21 @@ class HTTPConnection:
         self.putrequest(method, url, **skips)
 
         if body and ('content-length' not in header_names):
-            self.putheader('Content-Length', str(len(body)))
+            thelen=None
+            try:
+                thelen=str(len(body))
+            except TypeError, te:
+                # If this is a file-like object, try to
+                # fstat its file descriptor
+                import os
+                try:
+                    thelen = str(os.fstat(body.fileno()).st_size)
+                except (AttributeError, OSError):
+                    # Don't send a length if this failed
+                    if self.debuglevel > 0: print "Cannot stat!!"
+                    
+            if thelen is not None:
+                self.putheader('Content-Length',thelen)
         for hdr, value in headers.iteritems():
             self.putheader(hdr, value)
         self.endheaders()
@@ -863,7 +894,8 @@ class HTTPConnection:
         #                  isclosed() status to become true.
         #
         if self.__state != _CS_REQ_SENT or self.__response:
-            raise ResponseNotReady()
+            msg = "State %s, Reponse %s" % (self.__state, self.__response)
+            raise ResponseNotReady(msg)
 
         if self.debuglevel > 0:
             response = self.response_class(self.sock, self.debuglevel,
@@ -878,14 +910,17 @@ class HTTPConnection:
         self.__state = _CS_IDLE
 
         if response.will_close:
-            # this effectively passes the connection to the response
-            self.close()
+            # Pass the socket to the response
+            self.sock = None
         else:
             # remember this, so we can tell when it is complete
             self.__response = response
 
         return response
 
+    def is_idle (self):
+        return self.__state == _CS_IDLE
+
 # The next several classes are used to define FakeSocket,a socket-like
 # interface to an SSL connection.
 
@@ -1049,7 +1084,7 @@ class FakeSocket(SharedSocketClient):
 
     def makefile(self, mode, bufsize=None):
         if mode != 'r' and mode != 'rb':
-            raise UnimplementedFileMode()
+            raise UnimplementedFileMode("invalid file mode %r" % mode)
         return SSLFile(self._shared, self._ssl, bufsize)
 
     def send(self, stuff, flags = 0):
@@ -1151,7 +1186,7 @@ class HTTP:
 
             ### should we keep this behavior? do people use it?
             # keep the socket open (as a file), and return it
-            self.file = self._conn.sock.makefile('rb', 0)
+            self.file = self._conn.sock.makefile('rb')
 
             # close our socket -- we want to restart after any protocol error
             self.close()
